@@ -7,7 +7,9 @@ import ast
 import sys
 
 
-UnaryOp_ops = {'not': ast.Not}
+UnaryOp_ops = {'not': ast.Not,
+               '+':   ast.UAdd,
+               '-':   ast.USub}
 
 BinOp_ops = {'+': ast.Add,
              '-': ast.Sub,
@@ -28,6 +30,8 @@ op_strings = {value: key
               for dictionary in [UnaryOp_ops, BinOp_ops, BoolOp_ops, Compare_ops]
               for key, value in dictionary.items()}
 
+flexible_ops = ('+', '-')
+
 
 class PythonASTToTextASTTransformer(ast.NodeVisitor):
     def visit_Module(self, node):
@@ -69,25 +73,42 @@ class PythonASTToTextASTTransformer(ast.NodeVisitor):
     def visit_Tuple(self, node):
         return self.visit_List(node)
 
+    def visit_Dict(self, node):
+        return self.make_composite_node_string('dict',
+                                               self.visit(ast.List(elts=node.keys)),
+                                               self.visit(ast.List(elts=node.values)))
+
     def visit_Attribute(self, node):
         return self.make_composite_node_string('attr', self.visit(node.value), repr(node.attr))
 
+    def visit_Subscript(self, node):
+        return self.make_composite_node_string('subscript',
+                                               self.visit(node.value),
+                                               self.visit(node.slice))
+
+    def visit_Index(self, node):
+        return self.visit(node.value)
+
     def visit_Call(self, node):
         return self.make_composite_node_string('call',
                                                self.visit(node.func),
                                                *[self.visit(arg) for arg in node.args])
 
+    def visit_IfExp(self, node):
+        return self.make_composite_node_string('if',
+                                               self.visit(node.test),
+                                               self.visit(node.body),
+                                               self.visit(node.orelse))
+
     def visit_UnaryOp(self, node):
-        if isinstance(node.op, ast.UAdd):
-            return self.visit(node.operand)
-        elif isinstance(node.op, ast.USub):
-            if isinstance(node.operand, ast.Num):
+        if (hasattr(ast, 'Constant') and isinstance(node.operand, ast.Constant)
+           or isinstance(node.operand, ast.Num)):
+            if isinstance(node.op, ast.UAdd):
+                return self.visit(node.operand)
+            elif isinstance(node.op, ast.USub):
                 return self.visit(ast.Num(n=-node.operand.n))
-            else:
-                raise SyntaxError('Unsupported unary - operand type: ' + str(type(node.operand)))
-        else:
-            return self.make_composite_node_string(op_strings[type(node.op)],
-                                                   self.visit(node.operand))
+        return self.make_composite_node_string(op_strings[type(node.op)],
+                                               self.visit(node.operand))
 
     def visit_BinOp(self, node):
         return self.make_composite_node_string(op_strings[type(node.op)],
@@ -211,6 +232,16 @@ class TextASTToPythonASTTransformer(lark.Transformer):
         if node_type == 'list':
             return ast.List(elts=fields, ctx=ast.Load())
 
+        if node_type == 'dict':
+            if len(fields) != 2:
+                raise SyntaxError('Dictionary node must have two fields; found '
+                                  + str(len(fields)))
+            for field_index in range(2):
+                if not isinstance(fields[field_index], ast.List):
+                    raise SyntaxError('Dictionary fields must be lists; found '
+                                      + str(type(fields[field_index])))
+            return ast.Dict(keys=fields[0].elts, values=fields[1].elts)
+
         elif node_type == 'attr':
             if len(fields) != 2:
                 raise SyntaxError('Attribute node must have two fields; found ' + str(len(fields)))
@@ -218,6 +249,11 @@ class TextASTToPythonASTTransformer(lark.Transformer):
                 raise SyntaxError('Attribute name must be a string; found ' + str(type(fields[1])))
             return ast.Attribute(value=fields[0], attr=fields[1].s, ctx=ast.Load())
 
+        elif node_type == 'subscript':
+            if len(fields) != 2:
+                raise SyntaxError('Subscript node must have two fields; found ' + str(len(fields)))
+            return ast.Subscript(value=fields[0], slice=ast.Index(value=fields[1]), ctx=ast.Load())
+
         elif node_type == 'call':
             if len(fields) < 1:
                 raise SyntaxError('Call node must have at least one field; found '
@@ -231,19 +267,25 @@ class TextASTToPythonASTTransformer(lark.Transformer):
             else:
                 return ast.Call(func=fields[0], args=fields[1:], keywords=[])
 
+        elif node_type == 'if':
+            if len(fields) != 3:
+                raise SyntaxError('If node must have three fields; found '
+                                  + str(len(fields)))
+            return ast.IfExp(test=fields[0], body=fields[1], orelse=fields[2])
+
         elif node_type in UnaryOp_ops:
             if len(fields) == 1:
                 return ast.UnaryOp(op=UnaryOp_ops[node_type](), operand=fields[0])
-            else:
-                raise SyntaxError(UnaryOp_ops[node_type]
+            elif node_type not in flexible_ops:
+                raise SyntaxError(UnaryOp_ops[node_type].__name__
                                   + ' operator only supported for one operand; found '
                                   + str(len(fields)))
 
-        elif node_type in BinOp_ops:
+        if node_type in BinOp_ops:
             if len(fields) == 2:
                 return ast.BinOp(left=fields[0], op=BinOp_ops[node_type](), right=fields[1])
             else:
-                raise SyntaxError(BinOp_ops[node_type]
+                raise SyntaxError(BinOp_ops[node_type].__name__
                                   + ' operator only supported for two operands; found '
                                   + str(len(fields)))
 
@@ -251,7 +293,7 @@ class TextASTToPythonASTTransformer(lark.Transformer):
             if len(fields) == 2:
                 return ast.BoolOp(op=BoolOp_ops[node_type](), values=fields)
             else:
-                raise SyntaxError(BoolOp_ops[node_type]
+                raise SyntaxError(BoolOp_ops[node_type].__name__
                                   + ' operator only supported for two operands; found '
                                   + str(len(fields)))
 
@@ -261,7 +303,7 @@ class TextASTToPythonASTTransformer(lark.Transformer):
                                    ops=[Compare_ops[node_type]()],
                                    comparators=[fields[1]])
             else:
-                raise SyntaxError(Compare_ops[node_type]
+                raise SyntaxError(Compare_ops[node_type].__name__
                                   + ' operator only supported for two operands; found '
                                   + str(len(fields)))
 
